import { useState } from 'react'
import { Button, Flex, Modal, Select, Form, Input, Popover, Card } from 'antd'
import { QuestionCircleOutlined } from '@ant-design/icons'
import AppProgressSteps from '@/components/AppProgressSteps'
import { useDataFieldQuery } from '@/hooks/useQuery'
import { useAI } from '@/hooks/useAI'
import MarkdownRender from '@/components/MarkdownRender'

export default function ButtonModal() {
  const { fieldOptions } = useDataFieldQuery()

  const [open, setOpen] = useState(false)
  const [current, setCurrent] = useState(0)
  const [form] = Form.useForm()

  const { post, isLoading } = useAI()

  // AI设置
  const handleAISet = () => {
    const values = form.getFieldsValue()
    post(
      {
        response_format: { type: 'json_object' },
        messages: [
          {
            role: 'user',
            content: `根据提供的数据集，设计一个FP-Growth算法，返回JSON格式。
          自变量：${values.independent_variables.join(', ')}
          因变量：${values.dependent_variable}
          分别返回是否拟合截距项（fit_intercept）:true/false、是否复制数据（copy_x）:true/false、并行计算核心数（n_jobs）:None（单线程）或 ≥1 的整数（如 -1 表示使用所有核心）、强制正系数（positive）:true/false的值。
返回格式示例：
json
        {
          "results": { "fit_intercept": true, "copy_x": true, "n_jobs": 1, "positive": false}
        }
          `,
          },
        ],
      },
      {
        onComplete: (message) => {
          console.log(message)
          try {
            const parse = JSON.parse(message.content)
            form.setFieldsValue(parse.results)
          } catch (error) {
            console.error(error)
          }
        },
      }
    )
  }

  // AI计算
  const [results, setResults] = useState<any>({})
  const code = `${results.code?.replace(/\\n/gi, '\n')}`
  const [openResults, setOpenResults] = useState(false)
  const handleAICompute = () => {
    const values = form.getFieldsValue()
    post(
      {
        response_format: { type: 'json_object' },
        messages: [
          {
            role: 'user',
            content: `根据提供的数据集，设置的FP-Growth算法进行试算检查，返回JSON格式。
          自变量：${values.independent_variables.join(', ')}
          因变量：${values.dependent_variable}
          是否拟合截距项（fit_intercept）：${values.fit_intercept}
          是否复制数据（copy_X）：${values.copy_x}
          并行计算核心数（n_jobs）：${values.n_jobs}
          强制正系数（positive）：${values.positive}
          返回格式示例：
          json
          {
           results: { pass: 'true或者false，代表通过不通过', code: "Python运行的代码，markdown能解析的代码", result: "计算结果" },
          }
           `,
          },
        ],
      },
      {
        onComplete: (message) => {
          console.log(message)
          try {
            const parse = JSON.parse(message.content)
            setResults(parse.results)
          } catch (error) {
            console.error(error)
          }
          if (current === 1) {
            setStep(3)
          }
        },
      }
    )
  }

  const [step, setStep] = useState<number>(-1)

  const options = [
    { label: 'True', value: true },
    { label: 'False', value: false },
  ]

  // 处理下一步按钮逻辑
  const handleNext = async () => {
    await form.validateFields()
    setCurrent(current + 1)
    setStep(-1)
  }

  // 开始处理
  const handleStart = () => {
    setStep(1)
    handleAICompute()
  }

  // 关闭并重置
  const handleClose = () => {
    setOpen(false)
    setCurrent(0)
    setStep(-1)
  }

  const steps = [
    {
      title: '设置算法',
      content: (
        <>
          <Form.Item label="请选择特征列" name="independent_variables" rules={[{ required: true, message: '请选择' }]}>
            <Select options={fieldOptions} placeholder="请选择" mode="multiple" />
          </Form.Item>

          <Flex gap={20}>
            <div style={{ flex: 1 }}>
              <Form.Item label="请输入最小支持度（min_support）">
                <Flex gap={10}>
                  <Form.Item name="min_support" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请选择" />
                  </Form.Item>
                  <Popover
                    title="是否拟合截距项（fit_intercept）"
                    content={
                      <>
                        控制模型是否包含截距项（即偏置项，用于调整模型基线）
                        <br />
                        取值范围：
                        <br />
                        True（模型更灵活，适合大多数场景）
                        <br />
                        False（模型强制通过原点，可能欠拟合）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入最小置信度（min_confidence）">
                <Flex gap={10}>
                  <Form.Item name="min_confidence" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请选择" />
                  </Form.Item>
                  <Popover
                    title="是否复制数据（copy_X）"
                    content={
                      <>
                        决定是否复制输入数据（避免修改原始数据）
                        <br />
                        取值范围：
                        <br />
                        True（安全但内存占用高）
                        <br />
                        False（节省内存，但可能覆盖原始数据）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入最小提升度（min_lift）">
                <Flex gap={10}>
                  <Form.Item name="min_lift" noStyle rules={[{ required: true, message: '请输入' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="并行计算核心数（n_jobs）"
                    content={
                      <>
                        选择优化算法（不同算法支持不同正则化类型和计算效率）
                        <br />
                        取值范围：
                        <br />
                        None（单线程）或 ≥1 的整数（如 -1 表示使用所有核心）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入规则最小长度（min_length）">
                <Flex gap={10}>
                  <Form.Item name="min_length" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请选择" />
                  </Form.Item>
                  <Popover
                    title="强制正系数（positive）"
                    content={
                      <>
                        是否强制回归系数为非负值（适用于需正向解释的场景，如经济学或物理模型）
                        <br />
                        取值范围：
                        <br />
                        True（系数非负，可能牺牲模型拟合能力）
                        <br />
                        False（系数可正可负，灵活性高）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
            </div>
            <Button type="primary" style={{ height: '200px' }} onClick={handleAISet} loading={isLoading}>
              AI设置
            </Button>
          </Flex>
        </>
      ),
    },
    {
      title: 'FP-Growth处理',
      content: (
        <>
          <Flex vertical align="center" style={{ marginTop: '20px' }}>
            <Button type="primary" onClick={handleStart} loading={isLoading}>
              开始处理
            </Button>
            <AppProgressSteps current={step} items={[{ title: '启动' }, { title: '计算' }, { title: '计算结束' }]} />
          </Flex>
        </>
      ),
    },
  ]

  const Footer = () => {
    return (
      <Flex justify="center" gap={20}>
        {current === 0 && (
          <Button
            type="primary"
            onClick={() => {
              setOpenResults(true)
              handleAICompute()
            }}
            loading={isLoading}>
            AI检查试算
          </Button>
        )}
        {current > 0 && <Button onClick={() => setCurrent(current - 1)}>上一步</Button>}
        {current < steps.length - 1 && (
          <Button type="primary" onClick={handleNext}>
            下一步
          </Button>
        )}
        {current === steps.length - 1 && (
          <Button type="primary" onClick={handleClose} disabled={isLoading}>
            关闭
          </Button>
        )}
      </Flex>
    )
  }

  return (
    <>
      <Button type="primary" onClick={() => setOpen(true)}>
        FP-Growth
      </Button>
      <Modal
        title={steps[current].title}
        open={open}
        footer={<Footer />}
        destroyOnClose
        width={800}
        onCancel={handleClose}>
        <div style={{ minHeight: 300, padding: '20px 0' }}>
          <Form form={form} preserve={false}>
            {steps.map((item, index) => (
              <div key={index} hidden={current !== index} style={{ marginTop: '20px' }}>
                {item.content}
              </div>
            ))}
          </Form>
          {step >= 3 && (
            <Card style={{ marginTop: '20px' }}>
              <p>计算结束！</p>
              <p>计算结果如下：</p>
              {results.result}
            </Card>
          )}
        </div>
      </Modal>
      <Modal
        title="算法试算检查"
        open={openResults}
        footer={null}
        width={800}
        loading={isLoading}
        onCancel={() => setOpenResults(false)}>
        <p style={{ fontSize: 20, textAlign: 'center' }}>试算检查结果：{results.pass ? '通过' : '不通过'}！</p>
        <MarkdownRender>{code}</MarkdownRender>
      </Modal>
    </>
  )
}
